import { checkOptions } from './checkOptions';
import { createStore } from './createStore';
import { getAutocompleteSetters } from './getAutocompleteSetters';
import { getDefaultProps } from './getDefaultProps';
import { getPropGetters } from './getPropGetters';
import { onInput } from './onInput';
import { stateReducer } from './stateReducer';
export function createAutocomplete(options) {
  checkOptions(options);
  var subscribers = [];
  var props = getDefaultProps(options, subscribers);
  var store = createStore(stateReducer, props);

  var _getAutocompleteSette = getAutocompleteSetters({
    store: store
  }),
      setSelectedItemId = _getAutocompleteSette.setSelectedItemId,
      setQuery = _getAutocompleteSette.setQuery,
      setCollections = _getAutocompleteSette.setCollections,
      setIsOpen = _getAutocompleteSette.setIsOpen,
      setStatus = _getAutocompleteSette.setStatus,
      setContext = _getAutocompleteSette.setContext;

  var _getPropGetters = getPropGetters({
    store: store,
    props: props,
    setSelectedItemId: setSelectedItemId,
    setQuery: setQuery,
    setCollections: setCollections,
    setIsOpen: setIsOpen,
    setStatus: setStatus,
    setContext: setContext,
    refresh: refresh
  }),
      getEnvironmentProps = _getPropGetters.getEnvironmentProps,
      getRootProps = _getPropGetters.getRootProps,
      getFormProps = _getPropGetters.getFormProps,
      getLabelProps = _getPropGetters.getLabelProps,
      getInputProps = _getPropGetters.getInputProps,
      getPanelProps = _getPropGetters.getPanelProps,
      getListProps = _getPropGetters.getListProps,
      getItemProps = _getPropGetters.getItemProps;

  function refresh() {
    return onInput({
      query: store.getState().query,
      event: new Event('input'),
      store: store,
      props: props,
      setSelectedItemId: setSelectedItemId,
      setQuery: setQuery,
      setCollections: setCollections,
      setIsOpen: setIsOpen,
      setStatus: setStatus,
      setContext: setContext,
      nextState: {
        isOpen: store.getState().isOpen
      },
      refresh: refresh
    });
  }

  props.plugins.forEach(function (plugin) {
    var _plugin$subscribe;

    return (_plugin$subscribe = plugin.subscribe) === null || _plugin$subscribe === void 0 ? void 0 : _plugin$subscribe.call(plugin, {
      setSelectedItemId: setSelectedItemId,
      setQuery: setQuery,
      setCollections: setCollections,
      setIsOpen: setIsOpen,
      setStatus: setStatus,
      setContext: setContext,
      onSelect: function onSelect(fn) {
        subscribers.push({
          onSelect: fn
        });
      },
      onHighlight: function onHighlight(fn) {
        subscribers.push({
          onHighlight: fn
        });
      }
    });
  });
  return {
    setSelectedItemId: setSelectedItemId,
    setQuery: setQuery,
    setCollections: setCollections,
    setIsOpen: setIsOpen,
    setStatus: setStatus,
    setContext: setContext,
    getEnvironmentProps: getEnvironmentProps,
    getRootProps: getRootProps,
    getFormProps: getFormProps,
    getInputProps: getInputProps,
    getLabelProps: getLabelProps,
    getPanelProps: getPanelProps,
    getListProps: getListProps,
    getItemProps: getItemProps,
    refresh: refresh
  };
}