import { onInput } from './onInput';
import { getSelectedItem } from './utils';
export function onKeyDown(_ref) {
  var event = _ref.event,
      store = _ref.store,
      props = _ref.props,
      setSelectedItemId = _ref.setSelectedItemId,
      setQuery = _ref.setQuery,
      setCollections = _ref.setCollections,
      setIsOpen = _ref.setIsOpen,
      setStatus = _ref.setStatus,
      setContext = _ref.setContext,
      refresh = _ref.refresh;

  if (event.key === 'ArrowUp' || event.key === 'ArrowDown') {
    // Default browser behavior changes the caret placement on ArrowUp and
    // Arrow down.
    event.preventDefault();
    store.dispatch(event.key, null);
    var nodeItem = props.environment.document.getElementById("".concat(props.id, "-item-").concat(store.getState().selectedItemId));

    if (nodeItem) {
      if (nodeItem.scrollIntoViewIfNeeded) {
        nodeItem.scrollIntoViewIfNeeded(false);
      } else {
        nodeItem.scrollIntoView(false);
      }
    }

    var highlightedItem = getSelectedItem({
      state: store.getState()
    });

    if (store.getState().selectedItemId !== null && highlightedItem) {
      var item = highlightedItem.item,
          itemInputValue = highlightedItem.itemInputValue,
          itemUrl = highlightedItem.itemUrl,
          source = highlightedItem.source;
      source.onHighlight({
        item: item,
        itemInputValue: itemInputValue,
        itemUrl: itemUrl,
        source: source,
        state: store.getState(),
        setSelectedItemId: setSelectedItemId,
        setQuery: setQuery,
        setCollections: setCollections,
        setIsOpen: setIsOpen,
        setStatus: setStatus,
        setContext: setContext,
        event: event
      });
    }
  } else if (event.key === 'Escape') {
    // This prevents the default browser behavior on `input[type="search"]`
    // to remove the query right away because we first want to close the
    // panel.
    event.preventDefault();
    store.dispatch(event.key, null);
  } else if (event.key === 'Enter') {
    // No item is selected, so we let the browser handle the native `onSubmit`
    // form event.
    if (store.getState().selectedItemId === null || store.getState().collections.every(function (collection) {
      return collection.items.length === 0;
    })) {
      return;
    } // This prevents the `onSubmit` event to be sent because an item is
    // highlighted.


    event.preventDefault();

    var _ref2 = getSelectedItem({
      state: store.getState()
    }),
        _item = _ref2.item,
        _itemInputValue = _ref2.itemInputValue,
        _itemUrl = _ref2.itemUrl,
        _source = _ref2.source;

    if (event.metaKey || event.ctrlKey) {
      if (_itemUrl !== undefined) {
        _source.onSelect({
          item: _item,
          itemInputValue: _itemInputValue,
          itemUrl: _itemUrl,
          source: _source,
          state: store.getState(),
          setSelectedItemId: setSelectedItemId,
          setQuery: setQuery,
          setCollections: setCollections,
          setIsOpen: setIsOpen,
          setStatus: setStatus,
          setContext: setContext,
          event: event
        });

        props.navigator.navigateNewTab({
          itemUrl: _itemUrl,
          item: _item,
          state: store.getState()
        });
      }
    } else if (event.shiftKey) {
      if (_itemUrl !== undefined) {
        _source.onSelect({
          item: _item,
          itemInputValue: _itemInputValue,
          itemUrl: _itemUrl,
          source: _source,
          state: store.getState(),
          setSelectedItemId: setSelectedItemId,
          setQuery: setQuery,
          setCollections: setCollections,
          setIsOpen: setIsOpen,
          setStatus: setStatus,
          setContext: setContext,
          event: event
        });

        props.navigator.navigateNewWindow({
          itemUrl: _itemUrl,
          item: _item,
          state: store.getState()
        });
      }
    } else if (event.altKey) {// Keep native browser behavior
    } else {
      if (_itemUrl !== undefined) {
        _source.onSelect({
          item: _item,
          itemInputValue: _itemInputValue,
          itemUrl: _itemUrl,
          source: _source,
          state: store.getState(),
          setSelectedItemId: setSelectedItemId,
          setQuery: setQuery,
          setCollections: setCollections,
          setIsOpen: setIsOpen,
          setStatus: setStatus,
          setContext: setContext,
          event: event
        });

        props.navigator.navigate({
          itemUrl: _itemUrl,
          item: _item,
          state: store.getState()
        });
        return;
      }

      onInput({
        query: _itemInputValue,
        event: event,
        store: store,
        props: props,
        setSelectedItemId: setSelectedItemId,
        setQuery: setQuery,
        setCollections: setCollections,
        setIsOpen: setIsOpen,
        setStatus: setStatus,
        setContext: setContext,
        nextState: {
          isOpen: false
        },
        refresh: refresh
      }).then(function () {
        _source.onSelect({
          item: _item,
          itemInputValue: _itemInputValue,
          itemUrl: _itemUrl,
          source: _source,
          state: store.getState(),
          setSelectedItemId: setSelectedItemId,
          setQuery: setQuery,
          setCollections: setCollections,
          setIsOpen: setIsOpen,
          setStatus: setStatus,
          setContext: setContext,
          event: event
        });
      });
    }
  }
}