/*! @algolia/autocomplete-core 1.0.0-alpha.36 | MIT License | © Algolia, Inc. and contributors | https://github.com/algolia/autocomplete.js */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['@algolia/autocomplete-core'] = {}));
}(this, (function (exports) { 'use strict';

  var warnCache = {
    current: {}
  };
  /**
   * Logs a warning if the condition is not met.
   * This is used to log issues in development environment only.
   */

  function warn(condition, message) {

    if (condition) {
      return;
    }

    var sanitizedMessage = message.trim();
    var hasAlreadyPrinted = warnCache.current[sanitizedMessage];

    if (!hasAlreadyPrinted) {
      warnCache.current[sanitizedMessage] = true; // eslint-disable-next-line no-console

      console.warn("[Autocomplete] ".concat(sanitizedMessage));
    }
  }

  function checkOptions(option) {
    "development" !== 'production' ? warn(!option.debug, 'The `debug` option is meant for development debugging and should not be used in production.') : void 0;
  }

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);

    if (Object.getOwnPropertySymbols) {
      var symbols = Object.getOwnPropertySymbols(object);
      if (enumerableOnly) symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
      keys.push.apply(keys, symbols);
    }

    return keys;
  }

  function _objectSpread2(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i] != null ? arguments[i] : {};

      if (i % 2) {
        ownKeys(Object(source), true).forEach(function (key) {
          _defineProperty(target, key, source[key]);
        });
      } else if (Object.getOwnPropertyDescriptors) {
        Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
      } else {
        ownKeys(Object(source)).forEach(function (key) {
          Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
        });
      }
    }

    return target;
  }

  function _objectWithoutPropertiesLoose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;

    for (i = 0; i < sourceKeys.length; i++) {
      key = sourceKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      target[key] = source[key];
    }

    return target;
  }

  function _objectWithoutProperties(source, excluded) {
    if (source == null) return {};

    var target = _objectWithoutPropertiesLoose(source, excluded);

    var key, i;

    if (Object.getOwnPropertySymbols) {
      var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

      for (i = 0; i < sourceSymbolKeys.length; i++) {
        key = sourceSymbolKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
        target[key] = source[key];
      }
    }

    return target;
  }

  function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
  }

  function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
  }

  function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter);
  }

  function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
  }

  function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;

    for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

    return arr2;
  }

  function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  function createStore(reducer, props) {
    var state = props.initialState;
    return {
      getState: function getState() {
        return state;
      },
      dispatch: function dispatch(action, payload) {
        var prevState = _objectSpread2({}, state);

        state = reducer(state, {
          type: action,
          props: props,
          payload: payload
        });
        props.onStateChange({
          state: state,
          prevState: prevState
        });
      }
    };
  }

  function flatten(values) {
    return values.reduce(function (a, b) {
      return a.concat(b);
    }, []);
  }

  var autocompleteId = 0;
  function generateAutocompleteId() {
    return "autocomplete-".concat(autocompleteId++);
  }

  function getItemsCount(state) {
    if (state.collections.length === 0) {
      return 0;
    }

    return state.collections.reduce(function (sum, collection) {
      return sum + collection.items.length;
    }, 0);
  }

  function getNextSelectedItemId(moveAmount, baseIndex, itemCount, defaultSelectedItemId) {
    // We allow circular keyboard navigation from the base index.
    // The base index can either be `null` (nothing is highlighted) or `0`
    // (the first item is highlighted).
    // The base index is allowed to get assigned `null` only if
    // `props.defaultSelectedItemId` is `null`. This pattern allows to "stop"
    // by the actual query before navigating to other suggestions as seen on
    // Google or Amazon.
    if (baseIndex === null && moveAmount < 0) {
      return itemCount - 1;
    }

    if (defaultSelectedItemId !== null && baseIndex === 0 && moveAmount < 0) {
      return itemCount - 1;
    }

    var numericIndex = (baseIndex === null ? -1 : baseIndex) + moveAmount;

    if (numericIndex <= -1 || numericIndex >= itemCount) {
      return defaultSelectedItemId === null ? null : 0;
    }

    return numericIndex;
  }

  var noop = function noop() {};

  function getNormalizedSources(getSources, options) {
    return Promise.resolve(getSources(options)).then(function (sources) {
      return Promise.all(sources.filter(Boolean).map(function (source) {
        var normalizedSource = _objectSpread2({
          getItemInputValue: function getItemInputValue(_ref) {
            var state = _ref.state;
            return state.query;
          },
          getItemUrl: function getItemUrl() {
            return undefined;
          },
          onSelect: function onSelect(_ref2) {
            var setIsOpen = _ref2.setIsOpen;
            setIsOpen(false);
          },
          onHighlight: noop
        }, source);

        return Promise.resolve(normalizedSource);
      }));
    });
  }

  // We don't have access to the autocomplete source when we call `onKeyDown`
  // or `onClick` because those are native browser events.
  // However, we can get the source from the suggestion index.
  function getCollectionFromSelectedItemId(_ref) {
    var state = _ref.state;
    // Given 3 sources with respectively 1, 2 and 3 suggestions: [1, 2, 3]
    // We want to get the accumulated counts:
    // [1, 1 + 2, 1 + 2 + 3] = [1, 3, 3 + 3] = [1, 3, 6]
    var accumulatedCollectionsCount = state.collections.map(function (collections) {
      return collections.items.length;
    }).reduce(function (acc, collectionsCount, index) {
      var previousValue = acc[index - 1] || 0;
      var nextValue = previousValue + collectionsCount;
      acc.push(nextValue);
      return acc;
    }, []); // Based on the accumulated counts, we can infer the index of the suggestion.

    var collectionIndex = accumulatedCollectionsCount.reduce(function (acc, current) {
      if (current <= state.selectedItemId) {
        return acc + 1;
      }

      return acc;
    }, 0);
    return state.collections[collectionIndex];
  }
  /**
   * Gets the highlighted index relative to a suggestion object (not the absolute
   * highlighted index).
   *
   * Example:
   *  [['a', 'b'], ['c', 'd', 'e'], ['f']]
   *                      ↑
   *         (absolute: 3, relative: 1)
   */


  function getRelativeSelectedItemId(_ref2) {
    var state = _ref2.state,
        collection = _ref2.collection;
    var isOffsetFound = false;
    var counter = 0;
    var previousItemsOffset = 0;

    while (isOffsetFound === false) {
      var currentCollection = state.collections[counter];

      if (currentCollection === collection) {
        isOffsetFound = true;
        break;
      }

      previousItemsOffset += currentCollection.items.length;
      counter++;
    }

    return state.selectedItemId - previousItemsOffset;
  }

  function getSelectedItem(_ref3) {
    var state = _ref3.state;
    var collection = getCollectionFromSelectedItemId({
      state: state
    });

    if (!collection) {
      return null;
    }

    var item = collection.items[getRelativeSelectedItemId({
      state: state,
      collection: collection
    })];
    var source = collection.source;
    var itemInputValue = source.getItemInputValue({
      item: item,
      state: state
    });
    var itemUrl = source.getItemUrl({
      item: item,
      state: state
    });
    return {
      item: item,
      itemInputValue: itemInputValue,
      itemUrl: itemUrl,
      source: source
    };
  }

  function isOrContainsNode(parent, child) {
    return parent === child || parent.contains && parent.contains(child);
  }

  function getAutocompleteSetters(_ref) {
    var store = _ref.store;

    var setSelectedItemId = function setSelectedItemId(value) {
      store.dispatch('setSelectedItemId', value);
    };

    var setQuery = function setQuery(value) {
      store.dispatch('setQuery', value);
    };

    var setCollections = function setCollections(rawValue) {
      var baseItemId = 0;
      var value = rawValue.map(function (collection) {
        return _objectSpread2(_objectSpread2({}, collection), {}, {
          // We flatten the stored items to support calling `getAlgoliaHits`
          // from the source itself.
          items: flatten(collection.items).map(function (item) {
            return _objectSpread2(_objectSpread2({}, item), {}, {
              __autocomplete_id: baseItemId++
            });
          })
        });
      });
      store.dispatch('setCollections', value);
    };

    var setIsOpen = function setIsOpen(value) {
      store.dispatch('setIsOpen', value);
    };

    var setStatus = function setStatus(value) {
      store.dispatch('setStatus', value);
    };

    var setContext = function setContext(value) {
      store.dispatch('setContext', value);
    };

    return {
      setSelectedItemId: setSelectedItemId,
      setQuery: setQuery,
      setCollections: setCollections,
      setIsOpen: setIsOpen,
      setStatus: setStatus,
      setContext: setContext
    };
  }

  function getNavigator(_ref) {
    var environment = _ref.environment;
    return {
      navigate: function navigate(_ref2) {
        var itemUrl = _ref2.itemUrl;
        environment.location.assign(itemUrl);
      },
      navigateNewTab: function navigateNewTab(_ref3) {
        var itemUrl = _ref3.itemUrl;
        var windowReference = environment.open(itemUrl, '_blank', 'noopener');

        if (windowReference) {
          windowReference.focus();
        }
      },
      navigateNewWindow: function navigateNewWindow(_ref4) {
        var itemUrl = _ref4.itemUrl;
        environment.open(itemUrl, '_blank', 'noopener');
      }
    };
  }

  function getDefaultProps(props, subscribers) {
    var _props$id;

    var environment = typeof window !== 'undefined' ? window : {};
    var plugins = props.plugins || [];
    return _objectSpread2(_objectSpread2({
      debug: false,
      openOnFocus: false,
      placeholder: '',
      autoFocus: false,
      defaultSelectedItemId: null,
      stallThreshold: 300,
      environment: environment,
      shouldPanelShow: function shouldPanelShow(_ref) {
        var state = _ref.state;
        return getItemsCount(state) > 0;
      }
    }, props), {}, {
      // Since `generateAutocompleteId` triggers a side effect (it increments
      // and internal counter), we don't want to execute it if unnecessary.
      id: (_props$id = props.id) !== null && _props$id !== void 0 ? _props$id : generateAutocompleteId(),
      // The following props need to be deeply defaulted.
      initialState: _objectSpread2({
        selectedItemId: null,
        query: '',
        completion: null,
        collections: [],
        isOpen: false,
        status: 'idle',
        context: {}
      }, props.initialState),
      plugins: plugins,
      onStateChange: function onStateChange(params) {
        var _props$onStateChange;

        (_props$onStateChange = props.onStateChange) === null || _props$onStateChange === void 0 ? void 0 : _props$onStateChange.call(props, params);
        plugins.forEach(function (plugin) {
          var _plugin$onStateChange;

          (_plugin$onStateChange = plugin.onStateChange) === null || _plugin$onStateChange === void 0 ? void 0 : _plugin$onStateChange.call(plugin, params);
        });
      },
      onSubmit: function onSubmit(params) {
        var _props$onSubmit;

        (_props$onSubmit = props.onSubmit) === null || _props$onSubmit === void 0 ? void 0 : _props$onSubmit.call(props, params);
        plugins.forEach(function (plugin) {
          var _plugin$onSubmit;

          (_plugin$onSubmit = plugin.onSubmit) === null || _plugin$onSubmit === void 0 ? void 0 : _plugin$onSubmit.call(plugin, params);
        });
      },
      getSources: function getSources(options) {
        return Promise.all([].concat(_toConsumableArray(plugins.map(function (plugin) {
          return plugin.getSources;
        })), [props.getSources]).filter(Boolean).map(function (getSources) {
          return getNormalizedSources(getSources, options);
        })).then(function (nested) {
          return flatten(nested);
        }).then(function (sources) {
          return sources.map(function (source) {
            return _objectSpread2(_objectSpread2({}, source), {}, {
              onSelect: function onSelect(params) {
                source.onSelect(params);
                subscribers.forEach(function (subscriber) {
                  var _subscriber$onSelect;

                  (_subscriber$onSelect = subscriber.onSelect) === null || _subscriber$onSelect === void 0 ? void 0 : _subscriber$onSelect.call(subscriber, params);
                });
              },
              onHighlight: function onHighlight(params) {
                source.onHighlight(params);
                subscribers.forEach(function (subscriber) {
                  var _subscriber$onHighlig;

                  (_subscriber$onHighlig = subscriber.onHighlight) === null || _subscriber$onHighlig === void 0 ? void 0 : _subscriber$onHighlig.call(subscriber, params);
                });
              }
            });
          });
        });
      },
      navigator: _objectSpread2(_objectSpread2({}, getNavigator({
        environment: environment
      })), props.navigator)
    });
  }

  var lastStalledId = null;
  function onInput(_ref) {
    var query = _ref.query,
        event = _ref.event,
        store = _ref.store,
        props = _ref.props,
        setSelectedItemId = _ref.setSelectedItemId,
        setQuery = _ref.setQuery,
        setCollections = _ref.setCollections,
        setIsOpen = _ref.setIsOpen,
        setStatus = _ref.setStatus,
        setContext = _ref.setContext,
        _ref$nextState = _ref.nextState,
        nextState = _ref$nextState === void 0 ? {} : _ref$nextState,
        refresh = _ref.refresh;

    if (props.onInput) {
      return Promise.resolve(props.onInput({
        query: query,
        state: store.getState(),
        setSelectedItemId: setSelectedItemId,
        setQuery: setQuery,
        setCollections: setCollections,
        setIsOpen: setIsOpen,
        setStatus: setStatus,
        setContext: setContext,
        refresh: refresh
      }));
    }

    if (lastStalledId) {
      clearTimeout(lastStalledId);
    }

    setQuery(query);
    setSelectedItemId(props.defaultSelectedItemId);

    if (query.length === 0 && props.openOnFocus === false) {
      var _nextState$isOpen;

      setStatus('idle');
      setCollections(store.getState().collections.map(function (collection) {
        return _objectSpread2(_objectSpread2({}, collection), {}, {
          items: []
        });
      }));
      setIsOpen((_nextState$isOpen = nextState.isOpen) !== null && _nextState$isOpen !== void 0 ? _nextState$isOpen : props.shouldPanelShow({
        state: store.getState()
      }));
      return Promise.resolve();
    }

    setStatus('loading');
    lastStalledId = props.environment.setTimeout(function () {
      setStatus('stalled');
    }, props.stallThreshold);
    return props.getSources({
      query: query,
      state: store.getState(),
      setSelectedItemId: setSelectedItemId,
      setQuery: setQuery,
      setCollections: setCollections,
      setIsOpen: setIsOpen,
      setStatus: setStatus,
      setContext: setContext,
      refresh: refresh
    }).then(function (sources) {
      setStatus('loading'); // @TODO: convert `Promise.all` to fetching strategy.

      return Promise.all(sources.map(function (source) {
        return Promise.resolve(source.getItems({
          query: query,
          state: store.getState(),
          setSelectedItemId: setSelectedItemId,
          setQuery: setQuery,
          setCollections: setCollections,
          setIsOpen: setIsOpen,
          setStatus: setStatus,
          setContext: setContext,
          refresh: refresh
        })).then(function (items) {
          return {
            source: source,
            items: items
          };
        });
      })).then(function (collections) {
        var _nextState$isOpen2;

        setStatus('idle');
        setCollections(collections);
        setIsOpen((_nextState$isOpen2 = nextState.isOpen) !== null && _nextState$isOpen2 !== void 0 ? _nextState$isOpen2 : query.length === 0 && props.openOnFocus || props.shouldPanelShow({
          state: store.getState()
        }));
        var highlightedItem = getSelectedItem({
          state: store.getState()
        });

        if (store.getState().selectedItemId !== null && highlightedItem) {
          var item = highlightedItem.item,
              itemInputValue = highlightedItem.itemInputValue,
              itemUrl = highlightedItem.itemUrl,
              source = highlightedItem.source;
          source.onHighlight({
            item: item,
            itemInputValue: itemInputValue,
            itemUrl: itemUrl,
            source: source,
            state: store.getState(),
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            event: event
          });
        }
      }).catch(function (error) {
        setStatus('error');
        throw error;
      }).finally(function () {
        if (lastStalledId) {
          clearTimeout(lastStalledId);
        }
      });
    });
  }

  function onKeyDown(_ref) {
    var event = _ref.event,
        store = _ref.store,
        props = _ref.props,
        setSelectedItemId = _ref.setSelectedItemId,
        setQuery = _ref.setQuery,
        setCollections = _ref.setCollections,
        setIsOpen = _ref.setIsOpen,
        setStatus = _ref.setStatus,
        setContext = _ref.setContext,
        refresh = _ref.refresh;

    if (event.key === 'ArrowUp' || event.key === 'ArrowDown') {
      // Default browser behavior changes the caret placement on ArrowUp and
      // Arrow down.
      event.preventDefault();
      store.dispatch(event.key, null);
      var nodeItem = props.environment.document.getElementById("".concat(props.id, "-item-").concat(store.getState().selectedItemId));

      if (nodeItem) {
        if (nodeItem.scrollIntoViewIfNeeded) {
          nodeItem.scrollIntoViewIfNeeded(false);
        } else {
          nodeItem.scrollIntoView(false);
        }
      }

      var highlightedItem = getSelectedItem({
        state: store.getState()
      });

      if (store.getState().selectedItemId !== null && highlightedItem) {
        var item = highlightedItem.item,
            itemInputValue = highlightedItem.itemInputValue,
            itemUrl = highlightedItem.itemUrl,
            source = highlightedItem.source;
        source.onHighlight({
          item: item,
          itemInputValue: itemInputValue,
          itemUrl: itemUrl,
          source: source,
          state: store.getState(),
          setSelectedItemId: setSelectedItemId,
          setQuery: setQuery,
          setCollections: setCollections,
          setIsOpen: setIsOpen,
          setStatus: setStatus,
          setContext: setContext,
          event: event
        });
      }
    } else if (event.key === 'Escape') {
      // This prevents the default browser behavior on `input[type="search"]`
      // to remove the query right away because we first want to close the
      // panel.
      event.preventDefault();
      store.dispatch(event.key, null);
    } else if (event.key === 'Enter') {
      // No item is selected, so we let the browser handle the native `onSubmit`
      // form event.
      if (store.getState().selectedItemId === null || store.getState().collections.every(function (collection) {
        return collection.items.length === 0;
      })) {
        return;
      } // This prevents the `onSubmit` event to be sent because an item is
      // highlighted.


      event.preventDefault();

      var _ref2 = getSelectedItem({
        state: store.getState()
      }),
          _item = _ref2.item,
          _itemInputValue = _ref2.itemInputValue,
          _itemUrl = _ref2.itemUrl,
          _source = _ref2.source;

      if (event.metaKey || event.ctrlKey) {
        if (_itemUrl !== undefined) {
          _source.onSelect({
            item: _item,
            itemInputValue: _itemInputValue,
            itemUrl: _itemUrl,
            source: _source,
            state: store.getState(),
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            event: event
          });

          props.navigator.navigateNewTab({
            itemUrl: _itemUrl,
            item: _item,
            state: store.getState()
          });
        }
      } else if (event.shiftKey) {
        if (_itemUrl !== undefined) {
          _source.onSelect({
            item: _item,
            itemInputValue: _itemInputValue,
            itemUrl: _itemUrl,
            source: _source,
            state: store.getState(),
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            event: event
          });

          props.navigator.navigateNewWindow({
            itemUrl: _itemUrl,
            item: _item,
            state: store.getState()
          });
        }
      } else if (event.altKey) ; else {
        if (_itemUrl !== undefined) {
          _source.onSelect({
            item: _item,
            itemInputValue: _itemInputValue,
            itemUrl: _itemUrl,
            source: _source,
            state: store.getState(),
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            event: event
          });

          props.navigator.navigate({
            itemUrl: _itemUrl,
            item: _item,
            state: store.getState()
          });
          return;
        }

        onInput({
          query: _itemInputValue,
          event: event,
          store: store,
          props: props,
          setSelectedItemId: setSelectedItemId,
          setQuery: setQuery,
          setCollections: setCollections,
          setIsOpen: setIsOpen,
          setStatus: setStatus,
          setContext: setContext,
          nextState: {
            isOpen: false
          },
          refresh: refresh
        }).then(function () {
          _source.onSelect({
            item: _item,
            itemInputValue: _itemInputValue,
            itemUrl: _itemUrl,
            source: _source,
            state: store.getState(),
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            event: event
          });
        });
      }
    }
  }

  function getPropGetters(_ref) {
    var store = _ref.store,
        props = _ref.props,
        setSelectedItemId = _ref.setSelectedItemId,
        setQuery = _ref.setQuery,
        setCollections = _ref.setCollections,
        setIsOpen = _ref.setIsOpen,
        setStatus = _ref.setStatus,
        setContext = _ref.setContext,
        refresh = _ref.refresh;

    var getEnvironmentProps = function getEnvironmentProps(getterProps) {
      return {
        // On touch devices, we do not rely on the native `blur` event of the
        // input to close the panel, but rather on a custom `touchstart` event
        // outside of the autocomplete elements.
        // This ensures a working experience on mobile because we blur the input
        // on touch devices when the user starts scrolling (`touchmove`).
        onTouchStart: function onTouchStart(event) {
          if (store.getState().isOpen === false || event.target === getterProps.inputElement) {
            return;
          }

          var isTargetWithinAutocomplete = [getterProps.formElement, getterProps.panelElement].some(function (contextNode) {
            return contextNode && (isOrContainsNode(contextNode, event.target) || isOrContainsNode(contextNode, props.environment.document.activeElement));
          });

          if (isTargetWithinAutocomplete === false) {
            store.dispatch('blur', null);
          }
        },
        // When scrolling on touch devices (mobiles, tablets, etc.), we want to
        // mimic the native platform behavior where the input is blurred to
        // hide the virtual keyboard. This gives more vertical space to
        // discover all the suggestions showing up in the panel.
        onTouchMove: function onTouchMove(event) {
          if (store.getState().isOpen === false || getterProps.inputElement !== props.environment.document.activeElement || event.target === getterProps.inputElement) {
            return;
          }

          getterProps.inputElement.blur();
        }
      };
    };

    var getRootProps = function getRootProps(rest) {
      return _objectSpread2({
        role: 'combobox',
        'aria-expanded': store.getState().isOpen,
        'aria-haspopup': 'listbox',
        'aria-owns': store.getState().isOpen ? "".concat(props.id, "-list") : undefined,
        'aria-labelledby': "".concat(props.id, "-label")
      }, rest);
    };

    var getFormProps = function getFormProps(providedProps) {
      var inputElement = providedProps.inputElement,
          rest = _objectWithoutProperties(providedProps, ["inputElement"]);

      return _objectSpread2({
        action: '',
        noValidate: true,
        role: 'search',
        onSubmit: function onSubmit(event) {
          event.preventDefault();
          props.onSubmit({
            state: store.getState(),
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            event: event
          });
          store.dispatch('submit', null);

          if (providedProps.inputElement) {
            providedProps.inputElement.blur();
          }
        },
        onReset: function onReset(event) {
          event.preventDefault();

          if (props.openOnFocus) {
            onInput({
              query: '',
              event: event,
              store: store,
              props: props,
              setSelectedItemId: setSelectedItemId,
              setQuery: setQuery,
              setCollections: setCollections,
              setIsOpen: setIsOpen,
              setStatus: setStatus,
              setContext: setContext,
              refresh: refresh
            });
          }

          store.dispatch('reset', null);

          if (providedProps.inputElement) {
            providedProps.inputElement.focus();
          }
        }
      }, rest);
    };

    var getInputProps = function getInputProps(providedProps) {
      function onFocus(event) {
        // We want to trigger a query when `openOnFocus` is true
        // because the panel should open with the current query.
        if (props.openOnFocus || store.getState().query.length > 0) {
          onInput({
            query: store.getState().completion || store.getState().query,
            event: event,
            store: store,
            props: props,
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            refresh: refresh
          });
        }

        store.dispatch('focus', null);
      }

      var isTouchDevice = ('ontouchstart' in props.environment);

      var _ref2 = providedProps || {},
          inputElement = _ref2.inputElement,
          _ref2$maxLength = _ref2.maxLength,
          maxLength = _ref2$maxLength === void 0 ? 512 : _ref2$maxLength,
          rest = _objectWithoutProperties(_ref2, ["inputElement", "maxLength"]);

      return _objectSpread2({
        'aria-autocomplete': 'both',
        'aria-activedescendant': store.getState().isOpen && store.getState().selectedItemId !== null ? "".concat(props.id, "-item-").concat(store.getState().selectedItemId) : undefined,
        'aria-controls': store.getState().isOpen ? "".concat(props.id, "-list") : undefined,
        'aria-labelledby': "".concat(props.id, "-label"),
        value: store.getState().completion || store.getState().query,
        id: "".concat(props.id, "-input"),
        autoComplete: 'off',
        autoCorrect: 'off',
        autoCapitalize: 'off',
        spellCheck: 'false',
        autoFocus: props.autoFocus,
        placeholder: props.placeholder,
        maxLength: maxLength,
        type: 'search',
        onChange: function onChange(event) {
          onInput({
            query: event.currentTarget.value.slice(0, maxLength),
            event: event,
            store: store,
            props: props,
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            refresh: refresh
          });
        },
        onKeyDown: function onKeyDown$1(event) {
          onKeyDown({
            event: event,
            store: store,
            props: props,
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            refresh: refresh
          });
        },
        onFocus: onFocus,
        onBlur: function onBlur() {
          // We do rely on the `blur` event on touch devices.
          // See explanation in `onTouchStart`.
          if (!isTouchDevice) {
            store.dispatch('blur', null);
          }
        },
        onClick: function onClick(event) {
          // When the panel is closed and you click on the input while
          // the input is focused, the `onFocus` event is not triggered
          // (default browser behavior).
          // In an autocomplete context, it makes sense to open the panel in this
          // case.
          // We mimic this event by catching the `onClick` event which
          // triggers the `onFocus` for the panel to open.
          if (providedProps.inputElement === props.environment.document.activeElement && !store.getState().isOpen) {
            onFocus(event);
          }
        }
      }, rest);
    };

    var getLabelProps = function getLabelProps(rest) {
      return _objectSpread2({
        htmlFor: "".concat(props.id, "-input"),
        id: "".concat(props.id, "-label")
      }, rest);
    };

    var getListProps = function getListProps(rest) {
      return _objectSpread2({
        role: 'listbox',
        'aria-labelledby': "".concat(props.id, "-label"),
        id: "".concat(props.id, "-list")
      }, rest);
    };

    var getPanelProps = function getPanelProps(rest) {
      return _objectSpread2({
        onMouseDown: function onMouseDown(event) {
          // Prevents the `activeElement` from being changed to the panel so
          // that the blur event is not triggered, otherwise it closes the
          // panel.
          event.preventDefault();
        },
        onMouseLeave: function onMouseLeave() {
          store.dispatch('mouseleave', null);
        }
      }, rest);
    };

    var getItemProps = function getItemProps(providedProps) {
      var item = providedProps.item,
          source = providedProps.source,
          rest = _objectWithoutProperties(providedProps, ["item", "source"]);

      return _objectSpread2({
        id: "".concat(props.id, "-item-").concat(item.__autocomplete_id),
        role: 'option',
        'aria-selected': store.getState().selectedItemId === item.__autocomplete_id,
        onMouseMove: function onMouseMove(event) {
          if (item.__autocomplete_id === store.getState().selectedItemId) {
            return;
          }

          store.dispatch('mousemove', item.__autocomplete_id);
          var highlightedItem = getSelectedItem({
            state: store.getState()
          });

          if (store.getState().selectedItemId !== null && highlightedItem) {
            var _item = highlightedItem.item,
                itemInputValue = highlightedItem.itemInputValue,
                itemUrl = highlightedItem.itemUrl,
                _source = highlightedItem.source;

            _source.onHighlight({
              item: _item,
              itemInputValue: itemInputValue,
              itemUrl: itemUrl,
              source: _source,
              state: store.getState(),
              setSelectedItemId: setSelectedItemId,
              setQuery: setQuery,
              setCollections: setCollections,
              setIsOpen: setIsOpen,
              setStatus: setStatus,
              setContext: setContext,
              event: event
            });
          }
        },
        onMouseDown: function onMouseDown(event) {
          // Prevents the `activeElement` from being changed to the item so it
          // can remain with the current `activeElement`.
          event.preventDefault();
        },
        onClick: function onClick(event) {
          var itemInputValue = source.getItemInputValue({
            item: item,
            state: store.getState()
          });
          var itemUrl = source.getItemUrl({
            item: item,
            state: store.getState()
          }); // If `getItemUrl` is provided, it means that the suggestion
          // is a link, not plain text that aims at updating the query.
          // We can therefore skip the state change because it will update
          // the `selectedItemId`, resulting in a UI flash, especially
          // noticeable on mobile.

          var runPreCommand = itemUrl ? Promise.resolve() : onInput({
            query: itemInputValue,
            event: event,
            store: store,
            props: props,
            setSelectedItemId: setSelectedItemId,
            setQuery: setQuery,
            setCollections: setCollections,
            setIsOpen: setIsOpen,
            setStatus: setStatus,
            setContext: setContext,
            refresh: refresh,
            nextState: {
              isOpen: false
            }
          });
          runPreCommand.then(function () {
            source.onSelect({
              item: item,
              itemInputValue: itemInputValue,
              itemUrl: itemUrl,
              source: source,
              state: store.getState(),
              setSelectedItemId: setSelectedItemId,
              setQuery: setQuery,
              setCollections: setCollections,
              setIsOpen: setIsOpen,
              setStatus: setStatus,
              setContext: setContext,
              event: event
            });
          });
        }
      }, rest);
    };

    return {
      getEnvironmentProps: getEnvironmentProps,
      getRootProps: getRootProps,
      getFormProps: getFormProps,
      getLabelProps: getLabelProps,
      getInputProps: getInputProps,
      getPanelProps: getPanelProps,
      getListProps: getListProps,
      getItemProps: getItemProps
    };
  }

  function getCompletion(_ref) {
    var state = _ref.state;

    if (state.isOpen === false || state.selectedItemId === null) {
      return null;
    }

    var _ref2 = getSelectedItem({
      state: state
    }),
        itemInputValue = _ref2.itemInputValue;

    return itemInputValue || null;
  }

  var stateReducer = function stateReducer(state, action) {
    switch (action.type) {
      case 'setSelectedItemId':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            selectedItemId: action.payload
          });
        }

      case 'setQuery':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            query: action.payload,
            completion: null
          });
        }

      case 'setCollections':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            collections: action.payload
          });
        }

      case 'setIsOpen':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            isOpen: action.payload
          });
        }

      case 'setStatus':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            status: action.payload
          });
        }

      case 'setContext':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            context: _objectSpread2(_objectSpread2({}, state.context), action.payload)
          });
        }

      case 'ArrowDown':
        {
          var nextState = _objectSpread2(_objectSpread2({}, state), {}, {
            selectedItemId: getNextSelectedItemId(1, state.selectedItemId, getItemsCount(state), action.props.defaultSelectedItemId)
          });

          return _objectSpread2(_objectSpread2({}, nextState), {}, {
            completion: getCompletion({
              state: nextState
            })
          });
        }

      case 'ArrowUp':
        {
          var _nextState = _objectSpread2(_objectSpread2({}, state), {}, {
            selectedItemId: getNextSelectedItemId(-1, state.selectedItemId, getItemsCount(state), action.props.defaultSelectedItemId)
          });

          return _objectSpread2(_objectSpread2({}, _nextState), {}, {
            completion: getCompletion({
              state: _nextState
            })
          });
        }

      case 'Escape':
        {
          if (state.isOpen) {
            return _objectSpread2(_objectSpread2({}, state), {}, {
              isOpen: false,
              completion: null
            });
          }

          return _objectSpread2(_objectSpread2({}, state), {}, {
            query: '',
            status: 'idle',
            collections: []
          });
        }

      case 'submit':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            selectedItemId: null,
            isOpen: false,
            status: 'idle'
          });
        }

      case 'reset':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            selectedItemId: // Since we open the panel on reset when openOnFocus=true
            // we need to restore the highlighted index to the defaultSelectedItemId. (DocSearch use-case)
            // Since we close the panel when openOnFocus=false
            // we lose track of the highlighted index. (Query-suggestions use-case)
            action.props.openOnFocus === true ? action.props.defaultSelectedItemId : null,
            isOpen: action.props.openOnFocus,
            // @TODO: Check with UX team if we want to close the panel on reset.
            status: 'idle',
            query: ''
          });
        }

      case 'focus':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            selectedItemId: action.props.defaultSelectedItemId,
            isOpen: action.props.openOnFocus || state.query.length > 0
          });
        }

      case 'blur':
        {
          if (action.props.debug) {
            return state;
          }

          return _objectSpread2(_objectSpread2({}, state), {}, {
            isOpen: false,
            selectedItemId: null
          });
        }

      case 'mousemove':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            selectedItemId: action.payload
          });
        }

      case 'mouseleave':
        {
          return _objectSpread2(_objectSpread2({}, state), {}, {
            selectedItemId: action.props.defaultSelectedItemId
          });
        }

      default:
        return state;
    }
  };

  function createAutocomplete(options) {
    checkOptions(options);
    var subscribers = [];
    var props = getDefaultProps(options, subscribers);
    var store = createStore(stateReducer, props);

    var _getAutocompleteSette = getAutocompleteSetters({
      store: store
    }),
        setSelectedItemId = _getAutocompleteSette.setSelectedItemId,
        setQuery = _getAutocompleteSette.setQuery,
        setCollections = _getAutocompleteSette.setCollections,
        setIsOpen = _getAutocompleteSette.setIsOpen,
        setStatus = _getAutocompleteSette.setStatus,
        setContext = _getAutocompleteSette.setContext;

    var _getPropGetters = getPropGetters({
      store: store,
      props: props,
      setSelectedItemId: setSelectedItemId,
      setQuery: setQuery,
      setCollections: setCollections,
      setIsOpen: setIsOpen,
      setStatus: setStatus,
      setContext: setContext,
      refresh: refresh
    }),
        getEnvironmentProps = _getPropGetters.getEnvironmentProps,
        getRootProps = _getPropGetters.getRootProps,
        getFormProps = _getPropGetters.getFormProps,
        getLabelProps = _getPropGetters.getLabelProps,
        getInputProps = _getPropGetters.getInputProps,
        getPanelProps = _getPropGetters.getPanelProps,
        getListProps = _getPropGetters.getListProps,
        getItemProps = _getPropGetters.getItemProps;

    function refresh() {
      return onInput({
        query: store.getState().query,
        event: new Event('input'),
        store: store,
        props: props,
        setSelectedItemId: setSelectedItemId,
        setQuery: setQuery,
        setCollections: setCollections,
        setIsOpen: setIsOpen,
        setStatus: setStatus,
        setContext: setContext,
        nextState: {
          isOpen: store.getState().isOpen
        },
        refresh: refresh
      });
    }

    props.plugins.forEach(function (plugin) {
      var _plugin$subscribe;

      return (_plugin$subscribe = plugin.subscribe) === null || _plugin$subscribe === void 0 ? void 0 : _plugin$subscribe.call(plugin, {
        setSelectedItemId: setSelectedItemId,
        setQuery: setQuery,
        setCollections: setCollections,
        setIsOpen: setIsOpen,
        setStatus: setStatus,
        setContext: setContext,
        onSelect: function onSelect(fn) {
          subscribers.push({
            onSelect: fn
          });
        },
        onHighlight: function onHighlight(fn) {
          subscribers.push({
            onHighlight: fn
          });
        }
      });
    });
    return {
      setSelectedItemId: setSelectedItemId,
      setQuery: setQuery,
      setCollections: setCollections,
      setIsOpen: setIsOpen,
      setStatus: setStatus,
      setContext: setContext,
      getEnvironmentProps: getEnvironmentProps,
      getRootProps: getRootProps,
      getFormProps: getFormProps,
      getInputProps: getInputProps,
      getLabelProps: getLabelProps,
      getPanelProps: getPanelProps,
      getListProps: getListProps,
      getItemProps: getItemProps,
      refresh: refresh
    };
  }

  var version = '1.0.0-alpha.36';

  exports.createAutocomplete = createAutocomplete;
  exports.getDefaultProps = getDefaultProps;
  exports.version = version;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.development.js.map
